from .classify import (
    CategoryOverrides,
    ClassifyBuilderDocument,
    ClassifyDataset,
    ClassifyDatasetDocument,
    ClassifyDraftIteration,
    ClassifyDraftConfig,
    ClassifyIteration,
    ClassifyIterationDocument,
    ClassifyProject,
    ClassifyPublishedConfig,
    CreateClassifyProjectRequest,
    CreateClassifyDatasetRequest,
    CreateClassifyIterationRequest,
    PatchClassifyProjectRequest,
    PatchClassifyDatasetDocumentRequest,
    PatchClassifyDatasetRequest,
    PatchClassifyIterationRequest,
)
from .split import (
    CreateSplitProjectRequest,
    CreateSplitDatasetRequest,
    CreateSplitIterationRequest,
    PatchSplitProjectRequest,
    PatchSplitDatasetDocumentRequest,
    PatchSplitDatasetRequest,
    PatchSplitIterationRequest,
    SplitConfigOverrides,
    SplitBuilderDocument,
    SplitDataset,
    SplitDatasetDocument,
    SplitDraftConfig,
    SplitDraftIteration,
    SplitIteration,
    SplitIterationDocument,
    SplitProject,
    SplitPublishedConfig,
)

__all__ = [
    "CategoryOverrides",
    "ClassifyBuilderDocument",
    "ClassifyDataset",
    "ClassifyDatasetDocument",
    "ClassifyDraftConfig",
    "ClassifyDraftIteration",
    "ClassifyIteration",
    "ClassifyIterationDocument",
    "ClassifyProject",
    "ClassifyPublishedConfig",
    "CreateClassifyProjectRequest",
    "CreateClassifyDatasetRequest",
    "CreateClassifyIterationRequest",
    "CreateSplitProjectRequest",
    "CreateSplitDatasetRequest",
    "CreateSplitIterationRequest",
    "PatchClassifyProjectRequest",
    "PatchClassifyDatasetDocumentRequest",
    "PatchClassifyDatasetRequest",
    "PatchClassifyIterationRequest",
    "PatchSplitProjectRequest",
    "PatchSplitDatasetDocumentRequest",
    "PatchSplitDatasetRequest",
    "PatchSplitIterationRequest",
    "SplitConfigOverrides",
    "SplitBuilderDocument",
    "SplitDataset",
    "SplitDatasetDocument",
    "SplitDraftConfig",
    "SplitDraftIteration",
    "SplitIteration",
    "SplitIterationDocument",
    "SplitProject",
    "SplitPublishedConfig",
]



from typing import Any, Dict, List, Optional

from ...._resource import AsyncAPIResource, SyncAPIResource
from ....types.standards import PreparedRequest
from ....types.workflows import WorkflowEdge


class WorkflowEdgesMixin:
    """Mixin providing shared prepare methods for workflow edge operations."""

    def prepare_list(
        self,
        workflow_id: str,
        source_block: str | None = None,
        target_block: str | None = None,
    ) -> PreparedRequest:
        """Prepare a request to list all edges for a workflow."""
        params: Dict[str, Any] = {}
        if source_block is not None:
            params["source_block"] = source_block
        if target_block is not None:
            params["target_block"] = target_block
        return PreparedRequest(method="GET", url=f"/workflows/{workflow_id}/edges", params=params or None)

    def prepare_get(self, workflow_id: str, edge_id: str) -> PreparedRequest:
        """Prepare a request to get a single edge."""
        return PreparedRequest(method="GET", url=f"/workflows/{workflow_id}/edges/{edge_id}")

    def prepare_create(
        self,
        workflow_id: str,
        id: str,
        source_block: str,
        target_block: str,
        source_handle: str | None = None,
        target_handle: str | None = None,
    ) -> PreparedRequest:
        """Prepare a request to create a new edge."""
        data: Dict[str, Any] = {
            "id": id,
            "source_block": source_block,
            "target_block": target_block,
        }
        if source_handle is not None:
            data["source_handle"] = source_handle
        if target_handle is not None:
            data["target_handle"] = target_handle
        return PreparedRequest(method="POST", url=f"/workflows/{workflow_id}/edges", data=data)

    def prepare_create_batch(self, workflow_id: str, edges: List[Dict[str, Any]]) -> PreparedRequest:
        """Prepare a request to create multiple edges at once."""
        return PreparedRequest(method="POST", url=f"/workflows/{workflow_id}/edges/batch", data=edges)

    def prepare_delete(self, workflow_id: str, edge_id: str) -> PreparedRequest:
        """Prepare a request to delete an edge."""
        return PreparedRequest(method="DELETE", url=f"/workflows/{workflow_id}/edges/{edge_id}")

    def prepare_delete_all(self, workflow_id: str) -> PreparedRequest:
        """Prepare a request to delete all edges for a workflow."""
        return PreparedRequest(method="DELETE", url=f"/workflows/{workflow_id}/edges")


class WorkflowEdges(SyncAPIResource, WorkflowEdgesMixin):
    """Workflow Edges API wrapper for synchronous operations.

    Usage: ``client.workflows.edges.list(workflow_id)``
    """

    def list(
        self,
        workflow_id: str,

        source_block: str | None = None,
        target_block: str | None = None,
    ) -> List[WorkflowEdge]:
        """List all edges for a workflow.

        Args:
            workflow_id: The workflow ID
            source_block: Filter by source block ID (optional)
            target_block: Filter by target block ID (optional)

        Returns:
            List of workflow edges
        """
        request = self.prepare_list(workflow_id, source_block=source_block, target_block=target_block)
        response = self._client._prepared_request(request)
        return [WorkflowEdge.model_validate(item) for item in response]

    def get(self, workflow_id: str, edge_id: str) -> WorkflowEdge:
        """Get a single edge by ID."""
        request = self.prepare_get(workflow_id, edge_id)
        response = self._client._prepared_request(request)
        return WorkflowEdge.model_validate(response)

    def create(
        self,
        workflow_id: str,

        id: str,
        source_block: str,
        target_block: str,
        source_handle: str | None = None,
        target_handle: str | None = None,
    ) -> WorkflowEdge:
        """Create a new edge connecting two blocks.

        Args:
            workflow_id: The workflow ID
            id: Client-provided edge ID (e.g., "edge-1")
            source_block: Source block ID
            target_block: Target block ID
            source_handle: Output handle on source block (e.g., "output-file-0")
            target_handle: Input handle on target block (e.g., "input-file-0")

        Returns:
            WorkflowEdge: The created edge
        """
        request = self.prepare_create(
            workflow_id, id=id, source_block=source_block, target_block=target_block,
            source_handle=source_handle, target_handle=target_handle,
        )
        response = self._client._prepared_request(request)
        return WorkflowEdge.model_validate(response)

    def create_batch(self, workflow_id: str, edges: List[Dict[str, Any]]) -> List[WorkflowEdge]:
        """Create multiple edges in a single request.

        Args:
            workflow_id: The workflow ID
            edges: List of edge dicts, each with ``id``, ``source_block``, ``target_block``

        Returns:
            List of created edges
        """
        request = self.prepare_create_batch(workflow_id, edges)
        response = self._client._prepared_request(request)
        return [WorkflowEdge.model_validate(item) for item in response]

    def delete(self, workflow_id: str, edge_id: str) -> None:
        """Delete an edge."""
        request = self.prepare_delete(workflow_id, edge_id)
        self._client._prepared_request(request)

    def delete_all(self, workflow_id: str) -> None:
        """Delete all edges for a workflow."""
        request = self.prepare_delete_all(workflow_id)
        self._client._prepared_request(request)


class AsyncWorkflowEdges(AsyncAPIResource, WorkflowEdgesMixin):
    """Workflow Edges API wrapper for asynchronous operations.

    Usage: ``await client.workflows.edges.list(workflow_id)``
    """

    async def list(
        self,
        workflow_id: str,

        source_block: str | None = None,
        target_block: str | None = None,
    ) -> List[WorkflowEdge]:
        """List all edges for a workflow."""
        request = self.prepare_list(workflow_id, source_block=source_block, target_block=target_block)
        response = await self._client._prepared_request(request)
        return [WorkflowEdge.model_validate(item) for item in response]

    async def get(self, workflow_id: str, edge_id: str) -> WorkflowEdge:
        """Get a single edge by ID."""
        request = self.prepare_get(workflow_id, edge_id)
        response = await self._client._prepared_request(request)
        return WorkflowEdge.model_validate(response)

    async def create(
        self,
        workflow_id: str,

        id: str,
        source_block: str,
        target_block: str,
        source_handle: str | None = None,
        target_handle: str | None = None,
    ) -> WorkflowEdge:
        """Create a new edge connecting two blocks."""
        request = self.prepare_create(
            workflow_id, id=id, source_block=source_block, target_block=target_block,
            source_handle=source_handle, target_handle=target_handle,
        )
        response = await self._client._prepared_request(request)
        return WorkflowEdge.model_validate(response)

    async def create_batch(self, workflow_id: str, edges: List[Dict[str, Any]]) -> List[WorkflowEdge]:
        """Create multiple edges in a single request."""
        request = self.prepare_create_batch(workflow_id, edges)
        response = await self._client._prepared_request(request)
        return [WorkflowEdge.model_validate(item) for item in response]

    async def delete(self, workflow_id: str, edge_id: str) -> None:
        """Delete an edge."""
        request = self.prepare_delete(workflow_id, edge_id)
        await self._client._prepared_request(request)

    async def delete_all(self, workflow_id: str) -> None:
        """Delete all edges for a workflow."""
        request = self.prepare_delete_all(workflow_id)
        await self._client._prepared_request(request)



from typing import Any, Callable, Generic, Iterator, List, Literal, TypeVar

from pydantic import BaseModel, ConfigDict, PrivateAttr


T = TypeVar("T")


class ListMetadata(BaseModel):
    before: str | None
    after: str | None


class PaginatedList(BaseModel, Generic[T]):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    data: list[T]
    list_metadata: ListMetadata

    _fetch_next_page: Callable[..., "PaginatedList[T]"] | None = PrivateAttr(default=None)

    def __iter__(self) -> Iterator[T]:  # type: ignore[override]
        return iter(self.data)

    def __len__(self) -> int:
        return len(self.data)

    def __getitem__(self, index: int) -> T:
        return self.data[index]

    @property
    def has_more(self) -> bool:
        """Whether there are more pages available after this one."""
        return self.list_metadata.after is not None

    def auto_paging_iter(self) -> Iterator[T]:
        """Iterate through all items across all pages automatically.

        Yields items from the current page, then fetches subsequent pages
        until no more are available.
        """
        page = self
        while True:
            yield from page.data
            if not page.has_more or page._fetch_next_page is None:
                break
            page = page._fetch_next_page(after=page.list_metadata.after)


type PaginationOrder = Literal["asc", "desc"]


"""Typed exception hierarchy for the Retab SDK.

All API exceptions inherit from both :class:`APIError` and :class:`RuntimeError`,
so existing ``except RuntimeError`` blocks continue to catch them.
"""

from typing import Any


class RetabError(Exception):
    """Base class for all Retab SDK errors."""

    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(message)


class APIError(RetabError, RuntimeError):
    """A non-success HTTP response from the Retab API.

    Also inherits from :class:`RuntimeError` for backward compatibility
    with code that catches ``RuntimeError``.
    """

    def __init__(
        self,
        message: str,
        status_code: int,
        code: str | None = None,
        details: dict[str, Any] | None = None,
        body: str = "",
        request_id: str | None = None,
    ) -> None:
        self.status_code = status_code
        self.code = code
        self.details = details
        self.body = body
        self.request_id = request_id
        super().__init__(message)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(message={self.message!r}, "
            f"status_code={self.status_code}, code={self.code!r}, "
            f"request_id={self.request_id!r})"
        )


class AuthenticationError(APIError):
    """HTTP 401 — invalid or missing API key."""
    pass


class PermissionDeniedError(APIError):
    """HTTP 403 — valid API key but insufficient permissions."""
    pass


class NotFoundError(APIError):
    """HTTP 404 — the requested resource does not exist."""
    pass


class ValidationError(APIError):
    """HTTP 422 — request body failed server-side validation."""
    pass


class RateLimitError(APIError):
    """HTTP 429 — too many requests."""
    pass


class InternalServerError(APIError):
    """HTTP 5xx — server-side error."""
    pass


class APIConnectionError(RetabError):
    """Network-level error (DNS failure, refused connection, etc.)."""
    pass


class APITimeoutError(APIConnectionError):
    """The request timed out."""
    pass



"""Integration tests for the Jobs API across all supported endpoints.

Tests exercise the full job lifecycle: create → poll → verify result.
Run against local server:
    .venv/bin/python -m pytest --asyncio-mode=strict -W error -s -v --local test_jobs.py
"""

import base64
import time

import httpx
import pytest

from retab import Retab
from retab.types.jobs import Job

# ---------------------------------------------------------------------------
# Test data
# ---------------------------------------------------------------------------

INLINE_TEXT_DOCUMENT = {
    "filename": "test_invoice.txt",
    "url": "data:text/plain;base64,"
    + base64.b64encode(
        b"Invoice #12345\nDate: 2025-01-15\nAmount: $99.99\nCustomer: Acme Corp\nDescription: Consulting services"
    ).decode(),
}

SIMPLE_EXTRACT_SCHEMA = {
    "type": "object",
    "properties": {
        "invoice_number": {"type": "string", "description": "The invoice number"},
        "amount": {"type": "string", "description": "The total amount"},
        "customer": {"type": "string", "description": "The customer name"},
    },
    "required": ["invoice_number", "amount", "customer"],
}

MODEL = "retab-micro"

JOB_TIMEOUT = 120  # seconds
POLL_INTERVAL = 2  # seconds


# ---------------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------------


def _wait(client: Retab, job_id: str) -> Job:
    """Wait for a job to reach a terminal state and return it with full response."""
    return client.jobs.wait_for_completion(
        job_id,
        poll_interval_seconds=POLL_INTERVAL,
        timeout_seconds=JOB_TIMEOUT,
        include_response=True,
    )


def _assert_completed(job: Job) -> None:
    """Assert that a job completed successfully."""
    assert job.status == "completed", f"Expected completed, got {job.status}. Error: {job.error}"
    assert job.response is not None, "Completed job should have a response"
    assert job.response.status_code == 200, f"Expected 200, got {job.response.status_code}"


# ---------------------------------------------------------------------------
# Per-endpoint completion tests
# ---------------------------------------------------------------------------


def test_job_extract(sync_client: Retab) -> None:
    """Job for /v1/documents/extract completes and returns extracted data."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/extract",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "json_schema": SIMPLE_EXTRACT_SCHEMA,
                "model": MODEL,
            },
        )
        assert job.status in ("queued", "validating")

        job = _wait(client, job.id)
        _assert_completed(job)

        body = job.response.body
        assert "choices" in body, f"Extract response should have 'choices', got keys: {list(body.keys())}"


def test_job_parse(sync_client: Retab) -> None:
    """Job for /v1/documents/parse completes and returns parsed text."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
        )
        assert job.status in ("queued", "validating")

        job = _wait(client, job.id)
        _assert_completed(job)

        body = job.response.body
        assert "text" in body or "pages" in body, f"Parse response should have 'text' or 'pages', got keys: {list(body.keys())}"


def test_job_split(sync_client: Retab) -> None:
    """Job for /v1/documents/split completes and returns split result."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/split",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "subdocuments": [
                    {"name": "invoice_header", "description": "The header section with invoice number and date"},
                    {"name": "invoice_details", "description": "The details section with amount and customer"},
                ],
                "model": MODEL,
            },
        )
        assert job.status in ("queued", "validating")

        job = _wait(client, job.id)
        _assert_completed(job)

        body = job.response.body
        assert "splits" in body, f"Split response should have 'splits', got keys: {list(body.keys())}"
        assert isinstance(body["splits"], list), "splits should be a list"


def test_job_classify(sync_client: Retab) -> None:
    """Job for /v1/documents/classify completes and returns classification."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/classify",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "categories": [
                    {"name": "invoice", "description": "A billing invoice document"},
                    {"name": "receipt", "description": "A payment receipt"},
                    {"name": "contract", "description": "A legal contract"},
                ],
                "model": MODEL,
            },
        )
        assert job.status in ("queued", "validating")

        job = _wait(client, job.id)
        _assert_completed(job)

        body = job.response.body
        assert "result" in body, f"Classify response should have 'result', got keys: {list(body.keys())}"
        assert "classification" in body["result"], "result should contain 'classification'"


def test_job_schema_generate(sync_client: Retab) -> None:
    """Job for /v1/schemas/generate completes and returns a schema."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/schemas/generate",
            request={
                "documents": [INLINE_TEXT_DOCUMENT],
                "model": MODEL,
            },
        )
        assert job.status in ("queued", "validating")

        job = _wait(client, job.id)
        # Schema generation can fail with lightweight models producing invalid schemas
        assert job.status in ("completed", "failed"), f"Expected terminal state, got {job.status}"
        if job.status == "completed":
            assert job.response is not None
            assert isinstance(job.response.body, dict), "Schema generate should return a dict"


# ---------------------------------------------------------------------------
# Lifecycle tests
# ---------------------------------------------------------------------------


def test_job_create_returns_queued(sync_client: Retab) -> None:
    """Creating a job returns it with status 'queued'."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
        )
        assert job.id is not None
        assert job.status in ("queued", "validating")
        assert job.endpoint == "/v1/documents/parse"
        assert job.object == "job"

        # Clean up — wait for it to finish
        _wait(client, job.id)


def test_job_retrieve_without_payload(sync_client: Retab) -> None:
    """Default retrieve omits request and response payloads."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
        )
        # Wait for completion first
        _wait(client, job.id)

        # Retrieve without payloads (default)
        retrieved = client.jobs.retrieve(job.id)
        assert retrieved.id == job.id
        assert retrieved.status == "completed"
        assert retrieved.request is None, "Default retrieve should not include request"
        assert retrieved.response is None, "Default retrieve should not include response"


def test_job_retrieve_with_payload(sync_client: Retab) -> None:
    """retrieve_full includes both request and response."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
        )
        _wait(client, job.id)

        full = client.jobs.retrieve_full(job.id)
        assert full.id == job.id
        assert full.status == "completed"
        assert full.request is not None, "retrieve_full should include request"
        assert full.response is not None, "retrieve_full should include response"
        assert full.response.status_code == 200


def test_job_list_filters(sync_client: Retab) -> None:
    """List with endpoint and status filters returns matching jobs."""
    with sync_client as client:
        # Create and complete a parse job
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
        )
        _wait(client, job.id)

        # List with filters
        result = client.jobs.list(
            endpoint="/v1/documents/parse",
            status="completed",
            limit=5,
        )
        assert len(result.data) > 0, "Should find at least one completed parse job"
        for j in result.data:
            assert j.endpoint == "/v1/documents/parse"
            assert j.status == "completed"


def test_job_metadata_roundtrip(sync_client: Retab) -> None:
    """Metadata set at creation is returned in retrieve."""
    with sync_client as client:
        metadata = {"test_key": "test_value", "source": "sdk_test"}
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
            metadata=metadata,
        )
        assert job.metadata == metadata

        # Also verify after completion
        completed = _wait(client, job.id)
        retrieved = client.jobs.retrieve(completed.id)
        assert retrieved.metadata == metadata


def test_job_cancel(sync_client: Retab) -> None:
    """Cancelling a queued job sets status to 'cancelled'."""
    with sync_client as client:
        job = client.jobs.create(
            endpoint="/v1/documents/parse",
            request={
                "document": INLINE_TEXT_DOCUMENT,
                "model": MODEL,
            },
        )
        # Try to cancel immediately (may already be in_progress)
        try:
            cancelled = client.jobs.cancel(job.id)
            assert cancelled.status == "cancelled"
        except httpx.HTTPStatusError as e:
            # Job may have already completed or moved past cancellable state
            if e.response.status_code == 409:
                pass  # Expected if job already completed
            else:
                raise


def test_job_invalid_endpoint(sync_client: Retab) -> None:
    """Creating a job with an invalid endpoint raises an error."""
    with sync_client as client:
        with pytest.raises(Exception):
            client.jobs.create(
                endpoint="/v1/nonexistent/endpoint",  # type: ignore
                request={"document": INLINE_TEXT_DOCUMENT},
            )


def test_job_invalid_request_body(sync_client: Retab) -> None:
    """Creating a job with wrong request body shape raises a validation error."""
    with sync_client as client:
        with pytest.raises(Exception):
            client.jobs.create(
                endpoint="/v1/documents/extract",
                request={
                    # Missing required 'document' and 'json_schema'
                    "model": MODEL,
                },
            )



from __future__ import annotations

from typing import TYPE_CHECKING, List, Optional

from ....._resource import AsyncAPIResource, SyncAPIResource
from .....types.standards import PreparedRequest
from .....types.workflows import (
    StepOutputResponse,
    StepOutputsBatchResponse,
    WorkflowRunStep,
)

if TYPE_CHECKING:
    from .....types.workflows import WorkflowRun


class WorkflowStepsMixin:
    """Mixin providing shared prepare methods for workflow step operations."""

    def prepare_get(self, run_id: str, node_id: str) -> PreparedRequest:
        """Prepare a request to get status and handles for a specific step."""
        return PreparedRequest(method="GET", url=f"/workflows/runs/{run_id}/steps/{node_id}")

    def prepare_list(self, run_id: str, node_ids: Optional[List[str]] = None) -> PreparedRequest:
        """Prepare a request to list step documents, optionally filtered by node IDs."""
        if node_ids is not None:
            return PreparedRequest(
                method="POST",
                url=f"/workflows/runs/{run_id}/steps/batch",
                data={"node_ids": node_ids},
            )
        return PreparedRequest(method="GET", url=f"/workflows/runs/{run_id}/steps")


class WorkflowSteps(SyncAPIResource, WorkflowStepsMixin):
    """Workflow Run Steps API wrapper for synchronous operations.

    Usage: ``client.workflows.runs.steps.get(run_id, node_id)``
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get(self, run_id: str, node_id: str) -> StepOutputResponse:
        """Get step status and handle data for a specific step in a workflow run.

        Args:
            run_id: The ID of the workflow run
            node_id: The ID of the node/step to get output for

        Returns:
            StepOutputResponse with handle_outputs and handle_inputs.
                Use ``.extracted_data`` for quick access to JSON output data.
        """
        request = self.prepare_get(run_id, node_id)
        response = self._client._prepared_request(request)
        return StepOutputResponse.model_validate(response)

    def list(self, run_id: str, node_ids: Optional[List[str]] = None) -> List[WorkflowRunStep] | StepOutputsBatchResponse:
        """List step documents for a workflow run.

        Args:
            run_id: The ID of the workflow run
            node_ids: If provided, fetches only these specific nodes (batch mode).
                Returns ``StepOutputsBatchResponse`` keyed by node ID.
                If omitted, returns all steps as a list.

        Returns:
            List[WorkflowRunStep] when no node_ids given, or
            StepOutputsBatchResponse when node_ids are specified.
        """
        request = self.prepare_list(run_id, node_ids=node_ids)
        response = self._client._prepared_request(request)
        if node_ids is not None:
            return StepOutputsBatchResponse.model_validate(response)
        return [WorkflowRunStep.model_validate(item) for item in response]

    def get_all(self, run: WorkflowRun | str) -> StepOutputsBatchResponse:
        """Fetch outputs for all steps in a workflow run in one call.

        Args:
            run: A ``WorkflowRun`` object or a run ID string.
                If a string is passed, the run is fetched first to discover step node IDs.

        Returns:
            StepOutputsBatchResponse with all step outputs keyed by node ID.
        """
        if isinstance(run, str):
            from .....types.workflows import WorkflowRun as WR
            req = PreparedRequest(method="GET", url=f"/workflows/runs/{run}")
            resp = self._client._prepared_request(req)
            run = WR.model_validate(resp)
        node_ids = [s.node_id for s in run.steps]
        if not node_ids:
            return StepOutputsBatchResponse(outputs={})
        return self.list(run.id, node_ids=node_ids)


class AsyncWorkflowSteps(AsyncAPIResource, WorkflowStepsMixin):
    """Workflow Run Steps API wrapper for asynchronous operations.

    Usage: ``await client.workflows.runs.steps.get(run_id, node_id)``
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    async def get(self, run_id: str, node_id: str) -> StepOutputResponse:
        """Get step status and handle data for a specific step in a workflow run.

        Args:
            run_id: The ID of the workflow run
            node_id: The ID of the node/step to get output for

        Returns:
            StepOutputResponse with handle_outputs and handle_inputs.
                Use ``.extracted_data`` for quick access to JSON output data.
        """
        request = self.prepare_get(run_id, node_id)
        response = await self._client._prepared_request(request)
        return StepOutputResponse.model_validate(response)

    async def list(self, run_id: str, node_ids: Optional[List[str]] = None) -> List[WorkflowRunStep] | StepOutputsBatchResponse:
        """List step documents for a workflow run.

        Args:
            run_id: The ID of the workflow run
            node_ids: If provided, fetches only these specific nodes (batch mode).

        Returns:
            List[WorkflowRunStep] when no node_ids given, or
            StepOutputsBatchResponse when node_ids are specified.
        """
        request = self.prepare_list(run_id, node_ids=node_ids)
        response = await self._client._prepared_request(request)
        if node_ids is not None:
            return StepOutputsBatchResponse.model_validate(response)
        return [WorkflowRunStep.model_validate(item) for item in response]

    async def get_all(self, run: WorkflowRun | str) -> StepOutputsBatchResponse:
        """Fetch outputs for all steps in a workflow run in one call.

        Args:
            run: A ``WorkflowRun`` object or a run ID string.

        Returns:
            StepOutputsBatchResponse with all step outputs keyed by node ID.
        """
        if isinstance(run, str):
            from .....types.workflows import WorkflowRun as WR
            req = PreparedRequest(method="GET", url=f"/workflows/runs/{run}")
            resp = await self._client._prepared_request(req)
            run = WR.model_validate(resp)
        node_ids = [s.node_id for s in run.steps]
        if not node_ids:
            return StepOutputsBatchResponse(outputs={})
        return await self.list(run.id, node_ids=node_ids)



from io import IOBase
from pathlib import Path
from typing import Any, Dict

import PIL.Image
from pydantic import HttpUrl

from ...._resource import AsyncAPIResource, SyncAPIResource
from ....types.documents.classify import ClassifyResponse
from ....types.standards import PreparedRequest
from ....utils.mime import MIMEData
from ....types.evals.classify import ClassifyProject, CreateClassifyProjectRequest, PatchClassifyProjectRequest
from .datasets import AsyncDatasets, Datasets
from .templates import AsyncTemplates, Templates
from .._base import AsyncEvalCRUDMixin, SyncEvalCRUDMixin
from .._multipart import build_process_request


class ClassifyMixin:
    BASE = "/evals/classify"

    def prepare_create(
        self,
        name: str,
        categories: list[Any],
    ) -> PreparedRequest:
        project = CreateClassifyProjectRequest(name=name, categories=categories)
        return PreparedRequest(method="POST", url=self.BASE, data=project.model_dump(mode="json", exclude_unset=True))

    def prepare_update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> PreparedRequest:
        data = PatchClassifyProjectRequest(
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        ).model_dump(mode="json", exclude_unset=True, exclude_none=True)
        return PreparedRequest(method="PATCH", url=f"{self.BASE}/{eval_id}", data=data)

    def prepare_process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        documents: None = None,
        model: str | None = None,
        metadata: Dict[str, str] | None = None,
        **extra_form: Any,
    ) -> PreparedRequest:
        return build_process_request(
            base=self.BASE,
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            documents=documents,
            model=model,
            metadata=metadata,
            **extra_form,
        )


class Classify(SyncAPIResource, SyncEvalCRUDMixin, ClassifyMixin):
    """Classify evals API."""
    PROJECT_MODEL = ClassifyProject
    CREATE_MODEL = CreateClassifyProjectRequest
    PATCH_MODEL = PatchClassifyProjectRequest

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.datasets = Datasets(client=client)
        self.templates = Templates(client=client)

    def prepare_create(
        self,
        name: str,
        categories: list[Any],
    ) -> PreparedRequest:
        return ClassifyMixin.prepare_create(self, name=name, categories=categories)

    def prepare_update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> PreparedRequest:
        return ClassifyMixin.prepare_update(
            self,
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )

    def create(
        self,
        name: str,
        categories: list[Any],
    ) -> ClassifyProject:
        request = self.prepare_create(name=name, categories=categories)
        response = self._client._prepared_request(request)
        return ClassifyProject.model_validate(response)

    def update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> ClassifyProject:
        request = self.prepare_update(
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )
        response = self._client._prepared_request(request)
        return ClassifyProject.model_validate(response)

    def process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        metadata: Dict[str, str] | None = None,
        **extra_form: Any,
    ) -> ClassifyResponse:
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            metadata=metadata,
            **extra_form,
        )
        response = self._client._prepared_request(request)
        return ClassifyResponse.model_validate(response)


class AsyncClassify(AsyncAPIResource, AsyncEvalCRUDMixin, ClassifyMixin):
    """Async classify evals API."""
    PROJECT_MODEL = ClassifyProject
    CREATE_MODEL = CreateClassifyProjectRequest
    PATCH_MODEL = PatchClassifyProjectRequest

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.datasets = AsyncDatasets(client=client)
        self.templates = AsyncTemplates(client=client)

    def prepare_create(
        self,
        name: str,
        categories: list[Any],
    ) -> PreparedRequest:
        return ClassifyMixin.prepare_create(self, name=name, categories=categories)

    def prepare_update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> PreparedRequest:
        return ClassifyMixin.prepare_update(
            self,
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )

    async def create(
        self,
        name: str,
        categories: list[Any],
    ) -> ClassifyProject:
        request = self.prepare_create(name=name, categories=categories)
        response = await self._client._prepared_request(request)
        return ClassifyProject.model_validate(response)

    async def update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> ClassifyProject:
        request = self.prepare_update(
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )
        response = await self._client._prepared_request(request)
        return ClassifyProject.model_validate(response)

    async def process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        metadata: Dict[str, str] | None = None,
        **extra_form: Any,
    ) -> ClassifyResponse:
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            metadata=metadata,
            **extra_form,
        )
        response = await self._client._prepared_request(request)
        return ClassifyResponse.model_validate(response)



from typing import Any, List

from pydantic import BaseModel

from ...types.mime import BaseMIMEData
from ...types.pagination import ListMetadata, PaginatedList
from ...types.projects.model import BuilderDocument, Project
from ...types.standards import PreparedRequest


class TemplatesMixin:
    BASE: str

    def prepare_list(
        self,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
        fields: str | None = None,
    ) -> PreparedRequest:
        params: dict[str, Any] = {"limit": limit, "order": order}
        if before is not None:
            params["before"] = before
        if after is not None:
            params["after"] = after
        if fields is not None:
            params["fields"] = fields
        return PreparedRequest(method="GET", url=f"{self.BASE}/templates", params=params)

    def prepare_list_builder_document_previews(self, template_ids: List[str]) -> PreparedRequest:
        return PreparedRequest(
            method="GET",
            url=f"{self.BASE}/templates/builder-documents/previews",
            params={"template_ids": ",".join(template_ids)},
        )

    def prepare_get(self, template_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{self.BASE}/templates/{template_id}")

    def prepare_list_builder_documents(self, template_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{self.BASE}/templates/{template_id}/builder-documents")

    def prepare_clone(self, template_id: str, name: str | None = None) -> PreparedRequest:
        data = {"name": name} if name is not None else {}
        return PreparedRequest(method="POST", url=f"{self.BASE}/templates/{template_id}/clone", data=data)


class SyncTemplatesMixin(TemplatesMixin):
    PROJECT_MODEL: type[BaseModel] = Project
    BUILDER_DOCUMENT_MODEL: type[BaseModel] = BuilderDocument

    def list(
        self,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
        fields: str | None = None,
    ) -> PaginatedList[Any]:
        request = self.prepare_list(before=before, after=after, limit=limit, order=order, fields=fields)
        response = self._client._prepared_request(request)
        return PaginatedList(
            data=[self.PROJECT_MODEL.model_validate(item) for item in response.get("data", [])],
            list_metadata=ListMetadata.model_validate(response.get("list_metadata", {})),
        )

    def list_builder_document_previews(self, template_ids: List[str]) -> dict[str, List[BaseMIMEData]]:
        request = self.prepare_list_builder_document_previews(template_ids)
        response = self._client._prepared_request(request)
        return {
            template_id: [BaseMIMEData.model_validate(item) for item in previews]
            for template_id, previews in response.get("data", {}).items()
        }

    def get(self, template_id: str) -> BaseModel:
        request = self.prepare_get(template_id)
        response = self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    def list_builder_documents(self, template_id: str) -> List[BaseModel]:
        request = self.prepare_list_builder_documents(template_id)
        response = self._client._prepared_request(request)
        return [self.BUILDER_DOCUMENT_MODEL.model_validate(item) for item in response]

    def clone(self, template_id: str, name: str | None = None) -> BaseModel:
        request = self.prepare_clone(template_id, name=name)
        response = self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response["project"])


class AsyncTemplatesMixin(TemplatesMixin):
    PROJECT_MODEL: type[BaseModel] = Project
    BUILDER_DOCUMENT_MODEL: type[BaseModel] = BuilderDocument

    async def list(
        self,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
        fields: str | None = None,
    ) -> PaginatedList[Any]:
        request = self.prepare_list(before=before, after=after, limit=limit, order=order, fields=fields)
        response = await self._client._prepared_request(request)
        return PaginatedList(
            data=[self.PROJECT_MODEL.model_validate(item) for item in response.get("data", [])],
            list_metadata=ListMetadata.model_validate(response.get("list_metadata", {})),
        )

    async def list_builder_document_previews(self, template_ids: List[str]) -> dict[str, List[BaseMIMEData]]:
        request = self.prepare_list_builder_document_previews(template_ids)
        response = await self._client._prepared_request(request)
        return {
            template_id: [BaseMIMEData.model_validate(item) for item in previews]
            for template_id, previews in response.get("data", {}).items()
        }

    async def get(self, template_id: str) -> BaseModel:
        request = self.prepare_get(template_id)
        response = await self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    async def list_builder_documents(self, template_id: str) -> List[BaseModel]:
        request = self.prepare_list_builder_documents(template_id)
        response = await self._client._prepared_request(request)
        return [self.BUILDER_DOCUMENT_MODEL.model_validate(item) for item in response]

    async def clone(self, template_id: str, name: str | None = None) -> BaseModel:
        request = self.prepare_clone(template_id, name=name)
        response = await self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response["project"])



from typing import Any, List

from ....._resource import AsyncAPIResource, SyncAPIResource
from .....types.documents.extract import RetabParsedChatCompletion
from .....types.evals.split import (
    CreateSplitIterationRequest,
    PatchSplitIterationRequest,
    SplitConfigOverrides,
    SplitIteration,
    SplitIterationDocument,
)
from .....types.inference_settings import InferenceSettings
from .....types.projects.metrics import OptimizedIterationMetrics
from .....types.projects.predictions import PredictionData
from .....types.standards import PreparedRequest

BASE = "/evals/split"


class IterationsMixin:
    def prepare_create(
        self,
        eval_id: str,
        dataset_id: str,
        inference_settings: InferenceSettings | None = None,
        split_config_overrides: SplitConfigOverrides | None = None,
        parent_id: str | None = None,
    ) -> PreparedRequest:
        data = CreateSplitIterationRequest(
            inference_settings=inference_settings or InferenceSettings(),
            split_config_overrides=split_config_overrides or SplitConfigOverrides(),
            project_id=eval_id,
            dataset_id=dataset_id,
            parent_id=parent_id,
        ).model_dump(mode="json", exclude_unset=True, by_alias=True)
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations", data=data)

    def prepare_get(self, eval_id: str, dataset_id: str, iteration_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}")

    def prepare_list(
        self,
        eval_id: str,
        dataset_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> PreparedRequest:
        params: dict[str, Any] = {"limit": limit, "order": order}
        if before is not None:
            params["before"] = before
        if after is not None:
            params["after"] = after
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations", params=params)

    def prepare_update_draft(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        inference_settings: InferenceSettings | None = None,
        split_config_overrides: SplitConfigOverrides | None = None,
    ) -> PreparedRequest:
        data = PatchSplitIterationRequest(
            inference_settings=inference_settings,
            split_config_overrides=split_config_overrides,
        ).model_dump(mode="json", exclude_unset=True, by_alias=True)
        return PreparedRequest(method="PATCH", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}", data=data)

    def prepare_delete(self, eval_id: str, dataset_id: str, iteration_id: str) -> PreparedRequest:
        return PreparedRequest(method="DELETE", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}")

    def prepare_finalize(self, eval_id: str, dataset_id: str, iteration_id: str) -> PreparedRequest:
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/finalize")

    def prepare_get_schema(self, eval_id: str, dataset_id: str, iteration_id: str, use_draft: bool = False) -> PreparedRequest:
        params = {"use_draft": True} if use_draft else None
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/schema", params=params)

    def prepare_process_documents(self, eval_id: str, dataset_id: str, iteration_id: str, dataset_document_id: str) -> PreparedRequest:
        return PreparedRequest(
            method="POST",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/processDocumentsFromDatasetId",
            data={"dataset_document_id": dataset_document_id},
        )

    def prepare_get_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}")

    def prepare_list_documents(self, eval_id: str, dataset_id: str, iteration_id: str, limit: int = 1000, offset: int = 0) -> PreparedRequest:
        return PreparedRequest(
            method="GET",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents",
            params={"limit": limit, "offset": offset},
        )

    def prepare_update_document(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        document_id: str,
        prediction_data: Any | None = None,
        extraction_id: str | None = None,
    ) -> PreparedRequest:
        data: dict[str, Any] = {}
        if prediction_data is not None:
            data["prediction_data"] = prediction_data.model_dump(mode="json") if hasattr(prediction_data, "model_dump") else prediction_data
        if extraction_id is not None:
            data["extraction_id"] = extraction_id
        return PreparedRequest(
            method="PATCH",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}",
            data=data,
        )

    def prepare_delete_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(method="DELETE", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}")

    def prepare_get_metrics(self, eval_id: str, dataset_id: str, iteration_id: str, force_refresh: bool = False) -> PreparedRequest:
        params = {"force_refresh": True} if force_refresh else None
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/metrics", params=params)

    def prepare_process_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}/process")


class Iterations(SyncAPIResource, IterationsMixin):
    def create(
        self,
        eval_id: str,
        dataset_id: str,
        inference_settings: InferenceSettings | None = None,
        split_config_overrides: SplitConfigOverrides | None = None,
        parent_id: str | None = None,
    ) -> SplitIteration:
        response = self._client._prepared_request(self.prepare_create(eval_id, dataset_id, inference_settings, split_config_overrides, parent_id))
        return SplitIteration.model_validate(response)

    def get(self, eval_id: str, dataset_id: str, iteration_id: str) -> SplitIteration:
        response = self._client._prepared_request(self.prepare_get(eval_id, dataset_id, iteration_id))
        return SplitIteration.model_validate(response)

    def list(self, eval_id: str, dataset_id: str, before: str | None = None, after: str | None = None, limit: int = 10, order: str = "desc") -> List[SplitIteration]:
        response = self._client._prepared_request(self.prepare_list(eval_id, dataset_id, before=before, after=after, limit=limit, order=order))
        return [SplitIteration.model_validate(item) for item in response.get("data", [])]

    def update_draft(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        inference_settings: InferenceSettings | None = None,
        split_config_overrides: SplitConfigOverrides | None = None,
    ) -> SplitIteration:
        response = self._client._prepared_request(self.prepare_update_draft(eval_id, dataset_id, iteration_id, inference_settings, split_config_overrides))
        return SplitIteration.model_validate(response)

    def delete(self, eval_id: str, dataset_id: str, iteration_id: str) -> dict:
        return self._client._prepared_request(self.prepare_delete(eval_id, dataset_id, iteration_id))

    def finalize(self, eval_id: str, dataset_id: str, iteration_id: str) -> SplitIteration:
        response = self._client._prepared_request(self.prepare_finalize(eval_id, dataset_id, iteration_id))
        return SplitIteration.model_validate(response)

    def get_schema(self, eval_id: str, dataset_id: str, iteration_id: str, use_draft: bool = False) -> dict:
        return self._client._prepared_request(self.prepare_get_schema(eval_id, dataset_id, iteration_id, use_draft=use_draft))

    def process_documents(self, eval_id: str, dataset_id: str, iteration_id: str, dataset_document_id: str) -> dict:
        return self._client._prepared_request(self.prepare_process_documents(eval_id, dataset_id, iteration_id, dataset_document_id))

    def get_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> SplitIterationDocument:
        response = self._client._prepared_request(self.prepare_get_document(eval_id, dataset_id, iteration_id, document_id))
        return SplitIterationDocument.model_validate(response)

    def list_documents(self, eval_id: str, dataset_id: str, iteration_id: str, limit: int = 1000, offset: int = 0) -> List[SplitIterationDocument]:
        response = self._client._prepared_request(self.prepare_list_documents(eval_id, dataset_id, iteration_id, limit=limit, offset=offset))
        if isinstance(response, list):
            return [SplitIterationDocument.model_validate(item) for item in response]
        return [SplitIterationDocument.model_validate(item) for item in response.get("data", [])]

    def update_document(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        document_id: str,
        prediction_data: Any | None = None,
        extraction_id: str | None = None,
    ) -> SplitIterationDocument:
        response = self._client._prepared_request(self.prepare_update_document(eval_id, dataset_id, iteration_id, document_id, prediction_data=prediction_data, extraction_id=extraction_id))
        return SplitIterationDocument.model_validate(response)

    def delete_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> dict:
        return self._client._prepared_request(self.prepare_delete_document(eval_id, dataset_id, iteration_id, document_id))

    def get_metrics(self, eval_id: str, dataset_id: str, iteration_id: str, force_refresh: bool = False) -> OptimizedIterationMetrics:
        response = self._client._prepared_request(self.prepare_get_metrics(eval_id, dataset_id, iteration_id, force_refresh=force_refresh))
        return OptimizedIterationMetrics.model_validate(response)

    def process_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> RetabParsedChatCompletion:
        response = self._client._prepared_request(self.prepare_process_document(eval_id, dataset_id, iteration_id, document_id))
        return RetabParsedChatCompletion.model_validate(response)


class AsyncIterations(AsyncAPIResource, IterationsMixin):
    async def create(
        self,
        eval_id: str,
        dataset_id: str,
        inference_settings: InferenceSettings | None = None,
        split_config_overrides: SplitConfigOverrides | None = None,
        parent_id: str | None = None,
    ) -> SplitIteration:
        response = await self._client._prepared_request(self.prepare_create(eval_id, dataset_id, inference_settings, split_config_overrides, parent_id))
        return SplitIteration.model_validate(response)

    async def get(self, eval_id: str, dataset_id: str, iteration_id: str) -> SplitIteration:
        response = await self._client._prepared_request(self.prepare_get(eval_id, dataset_id, iteration_id))
        return SplitIteration.model_validate(response)

    async def list(self, eval_id: str, dataset_id: str, before: str | None = None, after: str | None = None, limit: int = 10, order: str = "desc") -> List[SplitIteration]:
        response = await self._client._prepared_request(self.prepare_list(eval_id, dataset_id, before=before, after=after, limit=limit, order=order))
        return [SplitIteration.model_validate(item) for item in response.get("data", [])]

    async def update_draft(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        inference_settings: InferenceSettings | None = None,
        split_config_overrides: SplitConfigOverrides | None = None,
    ) -> SplitIteration:
        response = await self._client._prepared_request(self.prepare_update_draft(eval_id, dataset_id, iteration_id, inference_settings, split_config_overrides))
        return SplitIteration.model_validate(response)

    async def delete(self, eval_id: str, dataset_id: str, iteration_id: str) -> dict:
        return await self._client._prepared_request(self.prepare_delete(eval_id, dataset_id, iteration_id))

    async def finalize(self, eval_id: str, dataset_id: str, iteration_id: str) -> SplitIteration:
        response = await self._client._prepared_request(self.prepare_finalize(eval_id, dataset_id, iteration_id))
        return SplitIteration.model_validate(response)

    async def get_schema(self, eval_id: str, dataset_id: str, iteration_id: str, use_draft: bool = False) -> dict:
        return await self._client._prepared_request(self.prepare_get_schema(eval_id, dataset_id, iteration_id, use_draft=use_draft))

    async def process_documents(self, eval_id: str, dataset_id: str, iteration_id: str, dataset_document_id: str) -> dict:
        return await self._client._prepared_request(self.prepare_process_documents(eval_id, dataset_id, iteration_id, dataset_document_id))

    async def get_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> SplitIterationDocument:
        response = await self._client._prepared_request(self.prepare_get_document(eval_id, dataset_id, iteration_id, document_id))
        return SplitIterationDocument.model_validate(response)

    async def list_documents(self, eval_id: str, dataset_id: str, iteration_id: str, limit: int = 1000, offset: int = 0) -> List[SplitIterationDocument]:
        response = await self._client._prepared_request(self.prepare_list_documents(eval_id, dataset_id, iteration_id, limit=limit, offset=offset))
        if isinstance(response, list):
            return [SplitIterationDocument.model_validate(item) for item in response]
        return [SplitIterationDocument.model_validate(item) for item in response.get("data", [])]

    async def update_document(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        document_id: str,
        prediction_data: Any | None = None,
        extraction_id: str | None = None,
    ) -> SplitIterationDocument:
        response = await self._client._prepared_request(self.prepare_update_document(eval_id, dataset_id, iteration_id, document_id, prediction_data=prediction_data, extraction_id=extraction_id))
        return SplitIterationDocument.model_validate(response)

    async def delete_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> dict:
        return await self._client._prepared_request(self.prepare_delete_document(eval_id, dataset_id, iteration_id, document_id))

    async def get_metrics(self, eval_id: str, dataset_id: str, iteration_id: str, force_refresh: bool = False) -> OptimizedIterationMetrics:
        response = await self._client._prepared_request(self.prepare_get_metrics(eval_id, dataset_id, iteration_id, force_refresh=force_refresh))
        return OptimizedIterationMetrics.model_validate(response)

    async def process_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> RetabParsedChatCompletion:
        response = await self._client._prepared_request(self.prepare_process_document(eval_id, dataset_id, iteration_id, document_id))
        return RetabParsedChatCompletion.model_validate(response)



from typing import Any, List

from ....._resource import AsyncAPIResource, SyncAPIResource
from .....types.evals.split import (
    CreateSplitDatasetRequest,
    PatchSplitDatasetDocumentRequest,
    PatchSplitDatasetRequest,
    SplitDataset,
    SplitDatasetDocument,
)
from .....types.inference_settings import InferenceSettings
from .....types.projects.predictions import PredictionData
from .....types.mime import MIMEData
from .....types.standards import PreparedRequest
from ..iterations import AsyncIterations, Iterations
from .....types.documents.extract import RetabParsedChatCompletion

BASE = "/evals/split"


class DatasetsMixin:
    def prepare_create(
        self,
        eval_id: str,
        name: str,
        base_split_config: list[dict[str, Any]] | list[Any] | None = None,
        base_inference_settings: InferenceSettings | None = None,
    ) -> PreparedRequest:
        data = CreateSplitDatasetRequest(
            name=name,
            base_split_config=base_split_config or [],
            base_inference_settings=base_inference_settings or InferenceSettings(),
        ).model_dump(mode="json", exclude_unset=True, by_alias=True)
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets", data=data)

    def prepare_get(self, eval_id: str, dataset_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}")

    def prepare_list(
        self,
        eval_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> PreparedRequest:
        params: dict[str, Any] = {"limit": limit, "order": order}
        if before is not None:
            params["before"] = before
        if after is not None:
            params["after"] = after
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets", params=params)

    def prepare_update(self, eval_id: str, dataset_id: str, name: str | None = None) -> PreparedRequest:
        data = PatchSplitDatasetRequest(name=name).model_dump(mode="json", exclude_unset=True)
        return PreparedRequest(method="PATCH", url=f"{BASE}/{eval_id}/datasets/{dataset_id}", data=data)

    def prepare_delete(self, eval_id: str, dataset_id: str) -> PreparedRequest:
        return PreparedRequest(method="DELETE", url=f"{BASE}/{eval_id}/datasets/{dataset_id}")

    def prepare_duplicate(self, eval_id: str, dataset_id: str, name: str | None = None) -> PreparedRequest:
        data = {"name": name} if name is not None else None
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/duplicate", data=data)

    def prepare_add_document(
        self,
        eval_id: str,
        dataset_id: str,
        mime_data: MIMEData,
        prediction_data: PredictionData | None = None,
    ) -> PreparedRequest:
        data: dict[str, Any] = {
            "mime_data": mime_data.model_dump(mode="json"),
            "project_id": eval_id,
            "dataset_id": dataset_id,
        }
        if prediction_data is not None:
            data["prediction_data"] = prediction_data.model_dump(mode="json")
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/dataset-documents", data=data)

    def prepare_get_document(self, eval_id: str, dataset_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/dataset-documents/{document_id}")

    def prepare_list_documents(self, eval_id: str, dataset_id: str, limit: int = 1000, offset: int = 0) -> PreparedRequest:
        return PreparedRequest(
            method="GET",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/dataset-documents",
            params={"limit": limit, "offset": offset},
        )

    def prepare_update_document(
        self,
        eval_id: str,
        dataset_id: str,
        document_id: str,
        validation_flags: dict[str, Any] | None = None,
        prediction_data: PredictionData | None = None,
        extraction_id: str | None = None,
    ) -> PreparedRequest:
        data = PatchSplitDatasetDocumentRequest(
            validation_flags=validation_flags,
            prediction_data=prediction_data,
            extraction_id=extraction_id,
        ).model_dump(mode="json", exclude_unset=True)
        return PreparedRequest(method="PATCH", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/dataset-documents/{document_id}", data=data)

    def prepare_delete_document(self, eval_id: str, dataset_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(method="DELETE", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/dataset-documents/{document_id}")

    def prepare_process_document(self, eval_id: str, dataset_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/dataset-documents/{document_id}/process")


class Datasets(SyncAPIResource, DatasetsMixin):
    def __init__(self, client: Any) -> None:
        super().__init__(client=client)
        self.iterations = Iterations(client=client)

    def create(
        self,
        eval_id: str,
        name: str,
        base_split_config: list[dict[str, Any]] | list[Any] | None = None,
        base_inference_settings: InferenceSettings | None = None,
    ) -> SplitDataset:
        response = self._client._prepared_request(self.prepare_create(eval_id, name, base_split_config, base_inference_settings))
        return SplitDataset.model_validate(response)

    def get(self, eval_id: str, dataset_id: str) -> SplitDataset:
        response = self._client._prepared_request(self.prepare_get(eval_id, dataset_id))
        return SplitDataset.model_validate(response)

    def list(
        self,
        eval_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> List[SplitDataset]:
        response = self._client._prepared_request(self.prepare_list(eval_id, before=before, after=after, limit=limit, order=order))
        return [SplitDataset.model_validate(item) for item in response.get("data", [])]

    def update(self, eval_id: str, dataset_id: str, name: str | None = None) -> SplitDataset:
        response = self._client._prepared_request(self.prepare_update(eval_id, dataset_id, name=name))
        return SplitDataset.model_validate(response)

    def delete(self, eval_id: str, dataset_id: str) -> dict:
        return self._client._prepared_request(self.prepare_delete(eval_id, dataset_id))

    def duplicate(self, eval_id: str, dataset_id: str, name: str | None = None) -> SplitDataset:
        response = self._client._prepared_request(self.prepare_duplicate(eval_id, dataset_id, name=name))
        return SplitDataset.model_validate(response)

    def add_document(self, eval_id: str, dataset_id: str, mime_data: MIMEData, prediction_data: PredictionData | None = None) -> SplitDatasetDocument:
        response = self._client._prepared_request(self.prepare_add_document(eval_id, dataset_id, mime_data, prediction_data=prediction_data))
        return SplitDatasetDocument.model_validate(response)

    def get_document(self, eval_id: str, dataset_id: str, document_id: str) -> SplitDatasetDocument:
        response = self._client._prepared_request(self.prepare_get_document(eval_id, dataset_id, document_id))
        return SplitDatasetDocument.model_validate(response)

    def list_documents(self, eval_id: str, dataset_id: str, limit: int = 1000, offset: int = 0) -> List[SplitDatasetDocument]:
        response = self._client._prepared_request(self.prepare_list_documents(eval_id, dataset_id, limit=limit, offset=offset))
        if isinstance(response, list):
            return [SplitDatasetDocument.model_validate(item) for item in response]
        return [SplitDatasetDocument.model_validate(item) for item in response.get("data", [])]

    def update_document(
        self,
        eval_id: str,
        dataset_id: str,
        document_id: str,
        validation_flags: dict[str, Any] | None = None,
        prediction_data: PredictionData | None = None,
        extraction_id: str | None = None,
    ) -> SplitDatasetDocument:
        response = self._client._prepared_request(
            self.prepare_update_document(
                eval_id,
                dataset_id,
                document_id,
                validation_flags=validation_flags,
                prediction_data=prediction_data,
                extraction_id=extraction_id,
            )
        )
        return SplitDatasetDocument.model_validate(response)

    def delete_document(self, eval_id: str, dataset_id: str, document_id: str) -> dict:
        return self._client._prepared_request(self.prepare_delete_document(eval_id, dataset_id, document_id))

    def process_document(self, eval_id: str, dataset_id: str, document_id: str) -> RetabParsedChatCompletion:
        response = self._client._prepared_request(self.prepare_process_document(eval_id, dataset_id, document_id))
        return RetabParsedChatCompletion.model_validate(response)


class AsyncDatasets(AsyncAPIResource, DatasetsMixin):
    def __init__(self, client: Any) -> None:
        super().__init__(client=client)
        self.iterations = AsyncIterations(client=client)

    async def create(
        self,
        eval_id: str,
        name: str,
        base_split_config: list[dict[str, Any]] | list[Any] | None = None,
        base_inference_settings: InferenceSettings | None = None,
    ) -> SplitDataset:
        response = await self._client._prepared_request(self.prepare_create(eval_id, name, base_split_config, base_inference_settings))
        return SplitDataset.model_validate(response)

    async def get(self, eval_id: str, dataset_id: str) -> SplitDataset:
        response = await self._client._prepared_request(self.prepare_get(eval_id, dataset_id))
        return SplitDataset.model_validate(response)

    async def list(
        self,
        eval_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> List[SplitDataset]:
        response = await self._client._prepared_request(self.prepare_list(eval_id, before=before, after=after, limit=limit, order=order))
        return [SplitDataset.model_validate(item) for item in response.get("data", [])]

    async def update(self, eval_id: str, dataset_id: str, name: str | None = None) -> SplitDataset:
        response = await self._client._prepared_request(self.prepare_update(eval_id, dataset_id, name=name))
        return SplitDataset.model_validate(response)

    async def delete(self, eval_id: str, dataset_id: str) -> dict:
        return await self._client._prepared_request(self.prepare_delete(eval_id, dataset_id))

    async def duplicate(self, eval_id: str, dataset_id: str, name: str | None = None) -> SplitDataset:
        response = await self._client._prepared_request(self.prepare_duplicate(eval_id, dataset_id, name=name))
        return SplitDataset.model_validate(response)

    async def add_document(self, eval_id: str, dataset_id: str, mime_data: MIMEData, prediction_data: PredictionData | None = None) -> SplitDatasetDocument:
        response = await self._client._prepared_request(self.prepare_add_document(eval_id, dataset_id, mime_data, prediction_data=prediction_data))
        return SplitDatasetDocument.model_validate(response)

    async def get_document(self, eval_id: str, dataset_id: str, document_id: str) -> SplitDatasetDocument:
        response = await self._client._prepared_request(self.prepare_get_document(eval_id, dataset_id, document_id))
        return SplitDatasetDocument.model_validate(response)

    async def list_documents(self, eval_id: str, dataset_id: str, limit: int = 1000, offset: int = 0) -> List[SplitDatasetDocument]:
        response = await self._client._prepared_request(self.prepare_list_documents(eval_id, dataset_id, limit=limit, offset=offset))
        if isinstance(response, list):
            return [SplitDatasetDocument.model_validate(item) for item in response]
        return [SplitDatasetDocument.model_validate(item) for item in response.get("data", [])]

    async def update_document(
        self,
        eval_id: str,
        dataset_id: str,
        document_id: str,
        validation_flags: dict[str, Any] | None = None,
        prediction_data: PredictionData | None = None,
        extraction_id: str | None = None,
    ) -> SplitDatasetDocument:
        response = await self._client._prepared_request(
            self.prepare_update_document(
                eval_id,
                dataset_id,
                document_id,
                validation_flags=validation_flags,
                prediction_data=prediction_data,
                extraction_id=extraction_id,
            )
        )
        return SplitDatasetDocument.model_validate(response)

    async def delete_document(self, eval_id: str, dataset_id: str, document_id: str) -> dict:
        return await self._client._prepared_request(self.prepare_delete_document(eval_id, dataset_id, document_id))

    async def process_document(self, eval_id: str, dataset_id: str, document_id: str) -> RetabParsedChatCompletion:
        response = await self._client._prepared_request(self.prepare_process_document(eval_id, dataset_id, document_id))
        return RetabParsedChatCompletion.model_validate(response)



from io import IOBase
from pathlib import Path
from typing import Any, Dict

import PIL.Image
from pydantic import HttpUrl

from ...._resource import AsyncAPIResource, SyncAPIResource
from ....types.documents.extract import RetabParsedChatCompletion
from ....types.standards import PreparedRequest
from ....utils.mime import MIMEData
from ....types.evals.split import CreateSplitProjectRequest, PatchSplitProjectRequest, SplitProject
from .datasets import AsyncDatasets, Datasets
from .templates import AsyncTemplates, Templates
from .._base import AsyncEvalCRUDMixin, SyncEvalCRUDMixin
from .._multipart import build_process_request


class SplitMixin:
    BASE = "/evals/split"

    def prepare_create(
        self,
        name: str,
        split_config: list[dict[str, Any]] | list[Any],
    ) -> PreparedRequest:
        project = CreateSplitProjectRequest(name=name, split_config=split_config)
        return PreparedRequest(method="POST", url=self.BASE, data=project.model_dump(mode="json", exclude_unset=True, by_alias=True))

    def prepare_update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> PreparedRequest:
        data = PatchSplitProjectRequest(
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        ).model_dump(mode="json", exclude_unset=True, exclude_none=True)
        return PreparedRequest(method="PATCH", url=f"{self.BASE}/{eval_id}", data=data)

    def prepare_process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        documents: None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> PreparedRequest:
        return build_process_request(
            base=self.BASE,
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            documents=documents,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )


class Split(SyncAPIResource, SyncEvalCRUDMixin, SplitMixin):
    """Split evals API."""
    PROJECT_MODEL = SplitProject
    CREATE_MODEL = CreateSplitProjectRequest
    PATCH_MODEL = PatchSplitProjectRequest

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.datasets = Datasets(client=client)
        self.templates = Templates(client=client)

    def prepare_create(
        self,
        name: str,
        split_config: list[dict[str, Any]] | list[Any],
    ) -> PreparedRequest:
        return SplitMixin.prepare_create(self, name=name, split_config=split_config)

    def prepare_update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> PreparedRequest:
        return SplitMixin.prepare_update(
            self,
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )

    def create(
        self,
        name: str,
        split_config: list[dict[str, Any]] | list[Any],
    ) -> SplitProject:
        request = self.prepare_create(name=name, split_config=split_config)
        response = self._client._prepared_request(request)
        return SplitProject.model_validate(response)

    def update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> SplitProject:
        request = self.prepare_update(
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )
        response = self._client._prepared_request(request)
        return SplitProject.model_validate(response)

    def process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        documents: None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> RetabParsedChatCompletion:
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            documents=documents,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        response = self._client._prepared_request(request)
        return RetabParsedChatCompletion.model_validate(response)


class AsyncSplit(AsyncAPIResource, AsyncEvalCRUDMixin, SplitMixin):
    """Async split evals API."""
    PROJECT_MODEL = SplitProject
    CREATE_MODEL = CreateSplitProjectRequest
    PATCH_MODEL = PatchSplitProjectRequest

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.datasets = AsyncDatasets(client=client)
        self.templates = AsyncTemplates(client=client)

    def prepare_create(
        self,
        name: str,
        split_config: list[dict[str, Any]] | list[Any],
    ) -> PreparedRequest:
        return SplitMixin.prepare_create(self, name=name, split_config=split_config)

    def prepare_update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> PreparedRequest:
        return SplitMixin.prepare_update(
            self,
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )

    async def create(
        self,
        name: str,
        split_config: list[dict[str, Any]] | list[Any],
    ) -> SplitProject:
        request = self.prepare_create(name=name, split_config=split_config)
        response = await self._client._prepared_request(request)
        return SplitProject.model_validate(response)

    async def update(
        self,
        eval_id: str,
        name: str | None = None,
        published_config: Any | None = None,
        draft_config: Any | None = None,
        is_published: bool | None = None,
    ) -> SplitProject:
        request = self.prepare_update(
            eval_id=eval_id,
            name=name,
            published_config=published_config,
            draft_config=draft_config,
            is_published=is_published,
        )
        response = await self._client._prepared_request(request)
        return SplitProject.model_validate(response)

    async def process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        documents: None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> RetabParsedChatCompletion:
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            documents=documents,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        response = await self._client._prepared_request(request)
        return RetabParsedChatCompletion.model_validate(response)



from typing import Any

from ..._resource import AsyncAPIResource, SyncAPIResource
from .classify import AsyncClassify, Classify
from .extract import AsyncExtract, Extract
from .split import AsyncSplit, Split


class Evals(SyncAPIResource):
    """Evals API — evaluation pipelines for extraction, split, and classification projects."""

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.extract = Extract(client=client)
        self.split = Split(client=client)
        self.classify = Classify(client=client)


class AsyncEvals(AsyncAPIResource):
    """Async Evals API."""

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.extract = AsyncExtract(client=client)
        self.split = AsyncSplit(client=client)
        self.classify = AsyncClassify(client=client)



import datetime
from typing import Any, Dict, List

from ....._resource import AsyncAPIResource, SyncAPIResource
from .....types.projects.iterations import (
    Iteration,
    IterationDocument,
    SchemaOverrides,
)
from .....types.projects.metrics import OptimizedIterationMetrics
from .....types.inference_settings import InferenceSettings
from .....types.standards import PreparedRequest
from .....types.documents.extract import RetabParsedChatCompletion

BASE = "/evals/extract"


class IterationsMixin:

    def prepare_create(
        self,
        eval_id: str,
        dataset_id: str,
        inference_settings: InferenceSettings | None = None,
        schema_overrides: SchemaOverrides | None = None,
        parent_id: str | None = None,
    ) -> PreparedRequest:
        data: Dict[str, Any] = {
            "project_id": eval_id,
            "dataset_id": dataset_id,
        }
        if inference_settings is not None:
            data["inference_settings"] = inference_settings.model_dump(mode="json")
        if schema_overrides is not None:
            data["schema_overrides"] = schema_overrides.model_dump(mode="json", exclude_none=True)
        if parent_id is not None:
            data["parent_id"] = parent_id
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations", data=data)

    def prepare_get(self, eval_id: str, dataset_id: str, iteration_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}")

    def prepare_list(
        self,
        eval_id: str,
        dataset_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> PreparedRequest:
        params: Dict[str, Any] = {"limit": limit, "order": order}
        if before is not None:
            params["before"] = before
        if after is not None:
            params["after"] = after
        return PreparedRequest(method="GET", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations", params=params)

    def prepare_update_draft(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        inference_settings: InferenceSettings | None = None,
        schema_overrides: SchemaOverrides | None = None,
    ) -> PreparedRequest:
        draft: Dict[str, Any] = {
            "updated_at": datetime.datetime.now(tz=datetime.timezone.utc).isoformat(),
        }
        if inference_settings is not None:
            draft["inference_settings"] = inference_settings.model_dump(mode="json")
        if schema_overrides is not None:
            draft["schema_overrides"] = schema_overrides.model_dump(mode="json", exclude_none=True)
        return PreparedRequest(
            method="PATCH",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}",
            data={"draft": draft},
        )

    def prepare_delete(self, eval_id: str, dataset_id: str, iteration_id: str) -> PreparedRequest:
        return PreparedRequest(method="DELETE", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}")

    def prepare_finalize(self, eval_id: str, dataset_id: str, iteration_id: str) -> PreparedRequest:
        return PreparedRequest(method="POST", url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/finalize")

    def prepare_get_schema(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        use_draft: bool = False,
    ) -> PreparedRequest:
        params: Dict[str, Any] = {}
        if use_draft:
            params["use_draft"] = True
        return PreparedRequest(
            method="GET",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/schema",
            params=params or None,
        )

    def prepare_process_documents(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        dataset_document_id: str,
    ) -> PreparedRequest:
        return PreparedRequest(
            method="POST",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/processDocumentsFromDatasetId",
            data={"dataset_document_id": dataset_document_id},
        )

    def prepare_get_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(
            method="GET",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}",
        )

    def prepare_list_documents(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        limit: int = 1000,
        offset: int = 0,
    ) -> PreparedRequest:
        params: Dict[str, Any] = {"limit": limit, "offset": offset}
        return PreparedRequest(
            method="GET",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents",
            params=params,
        )

    def prepare_update_document(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        document_id: str,
        prediction_data: Any | None = None,
        extraction_id: str | None = None,
    ) -> PreparedRequest:
        data: Dict[str, Any] = {}
        if prediction_data is not None:
            data["prediction_data"] = prediction_data.model_dump(mode="json") if hasattr(prediction_data, "model_dump") else prediction_data
        if extraction_id is not None:
            data["extraction_id"] = extraction_id
        return PreparedRequest(
            method="PATCH",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}",
            data=data,
        )

    def prepare_delete_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(
            method="DELETE",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}",
        )

    def prepare_get_metrics(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        force_refresh: bool = False,
    ) -> PreparedRequest:
        params: Dict[str, Any] = {}
        if force_refresh:
            params["force_refresh"] = True
        return PreparedRequest(
            method="GET",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/metrics",
            params=params or None,
        )

    def prepare_process_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> PreparedRequest:
        return PreparedRequest(
            method="POST",
            url=f"{BASE}/{eval_id}/datasets/{dataset_id}/iterations/{iteration_id}/iteration-documents/{document_id}/process",
        )


class Iterations(SyncAPIResource, IterationsMixin):

    def __init__(self, client: Any) -> None:
        super().__init__(client=client)

    def create(
        self,
        eval_id: str,
        dataset_id: str,
        inference_settings: InferenceSettings | None = None,
        schema_overrides: SchemaOverrides | None = None,
        parent_id: str | None = None,
    ) -> Iteration:
        request = self.prepare_create(eval_id=eval_id, dataset_id=dataset_id, inference_settings=inference_settings, schema_overrides=schema_overrides, parent_id=parent_id)
        response = self._client._prepared_request(request)
        return Iteration.model_validate(response)

    def get(self, eval_id: str, dataset_id: str, iteration_id: str) -> Iteration:
        request = self.prepare_get(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id)
        response = self._client._prepared_request(request)
        return Iteration.model_validate(response)

    def list(
        self,
        eval_id: str,
        dataset_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> List[Iteration]:
        request = self.prepare_list(eval_id=eval_id, dataset_id=dataset_id, before=before, after=after, limit=limit, order=order)
        response = self._client._prepared_request(request)
        return [Iteration.model_validate(item) for item in response.get("data", [])]

    def update_draft(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        inference_settings: InferenceSettings | None = None,
        schema_overrides: SchemaOverrides | None = None,
    ) -> Iteration:
        """Update the iteration's draft configuration.

        Iterations use a draft system: edits are staged in the draft sub-object.
        Call finalize() to promote the draft to the main iteration fields.
        Use get_schema(use_draft=True) to preview the effect of draft changes.
        """
        request = self.prepare_update_draft(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, inference_settings=inference_settings, schema_overrides=schema_overrides)
        response = self._client._prepared_request(request)
        return Iteration.model_validate(response)

    def delete(self, eval_id: str, dataset_id: str, iteration_id: str) -> dict:
        request = self.prepare_delete(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id)
        return self._client._prepared_request(request)

    def finalize(self, eval_id: str, dataset_id: str, iteration_id: str) -> Iteration:
        """Finalize a draft iteration: promotes draft values to main fields, sets status to 'completed'."""
        request = self.prepare_finalize(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id)
        response = self._client._prepared_request(request)
        return Iteration.model_validate(response)

    def get_schema(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        use_draft: bool = False,
    ) -> dict:
        """Get the materialized JSON schema for this iteration.

        Args:
            use_draft: If True, applies pending draft overrides on top of finalized overrides.
        """
        request = self.prepare_get_schema(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, use_draft=use_draft)
        return self._client._prepared_request(request)

    def process_documents(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        dataset_document_id: str,
    ) -> dict:
        """Trigger background extraction for an iteration document."""
        request = self.prepare_process_documents(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, dataset_document_id=dataset_document_id)
        return self._client._prepared_request(request)

    def get_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> IterationDocument:
        request = self.prepare_get_document(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, document_id=document_id)
        response = self._client._prepared_request(request)
        return IterationDocument.model_validate(response)

    def list_documents(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        limit: int = 1000,
        offset: int = 0,
    ) -> List[IterationDocument]:
        request = self.prepare_list_documents(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, limit=limit, offset=offset)
        response = self._client._prepared_request(request)
        if isinstance(response, list):
            return [IterationDocument.model_validate(item) for item in response]
        return [IterationDocument.model_validate(item) for item in response.get("data", [])]

    def update_document(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        document_id: str,
        prediction_data: Any | None = None,
        extraction_id: str | None = None,
    ) -> IterationDocument:
        request = self.prepare_update_document(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, document_id=document_id, prediction_data=prediction_data, extraction_id=extraction_id)
        response = self._client._prepared_request(request)
        return IterationDocument.model_validate(response)

    def delete_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> dict:
        request = self.prepare_delete_document(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, document_id=document_id)
        return self._client._prepared_request(request)

    def get_metrics(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        force_refresh: bool = False,
    ) -> OptimizedIterationMetrics:
        request = self.prepare_get_metrics(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, force_refresh=force_refresh)
        response = self._client._prepared_request(request)
        return OptimizedIterationMetrics.model_validate(response)

    def process_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> RetabParsedChatCompletion:
        request = self.prepare_process_document(eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id, document_id=document_id)
        response = self._client._prepared_request(request)
        return RetabParsedChatCompletion.model_validate(response)


class AsyncIterations(AsyncAPIResource, IterationsMixin):

    def __init__(self, client: Any) -> None:
        super().__init__(client=client)

    async def create(
        self,
        eval_id: str,
        dataset_id: str,
        inference_settings: InferenceSettings | None = None,
        schema_overrides: SchemaOverrides | None = None,
        parent_id: str | None = None,
    ) -> Iteration:
        request = self.prepare_create(eval_id, dataset_id, inference_settings=inference_settings, schema_overrides=schema_overrides, parent_id=parent_id)
        response = await self._client._prepared_request(request)
        return Iteration.model_validate(response)

    async def get(self, eval_id: str, dataset_id: str, iteration_id: str) -> Iteration:
        request = self.prepare_get(eval_id, dataset_id, iteration_id)
        response = await self._client._prepared_request(request)
        return Iteration.model_validate(response)

    async def list(
        self,
        eval_id: str,
        dataset_id: str,
        before: str | None = None,
        after: str | None = None,
        limit: int = 10,
        order: str = "desc",
    ) -> List[Iteration]:
        request = self.prepare_list(eval_id, dataset_id, before=before, after=after, limit=limit, order=order)
        response = await self._client._prepared_request(request)
        return [Iteration.model_validate(item) for item in response.get("data", [])]

    async def update_draft(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        inference_settings: InferenceSettings | None = None,
        schema_overrides: SchemaOverrides | None = None,
    ) -> Iteration:
        """Update the iteration's draft configuration."""
        request = self.prepare_update_draft(eval_id, dataset_id, iteration_id, inference_settings=inference_settings, schema_overrides=schema_overrides)
        response = await self._client._prepared_request(request)
        return Iteration.model_validate(response)

    async def delete(self, eval_id: str, dataset_id: str, iteration_id: str) -> dict:
        request = self.prepare_delete(eval_id, dataset_id, iteration_id)
        return await self._client._prepared_request(request)

    async def finalize(self, eval_id: str, dataset_id: str, iteration_id: str) -> Iteration:
        """Finalize a draft iteration: promotes draft values to main fields, sets status to 'completed'."""
        request = self.prepare_finalize(eval_id, dataset_id, iteration_id)
        response = await self._client._prepared_request(request)
        return Iteration.model_validate(response)

    async def get_schema(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        use_draft: bool = False,
    ) -> dict:
        """Get the materialized JSON schema for this iteration."""
        request = self.prepare_get_schema(eval_id, dataset_id, iteration_id, use_draft=use_draft)
        return await self._client._prepared_request(request)

    async def process_documents(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        dataset_document_id: str,
    ) -> dict:
        """Trigger background extraction for an iteration document."""
        request = self.prepare_process_documents(eval_id, dataset_id, iteration_id, dataset_document_id)
        return await self._client._prepared_request(request)

    async def get_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> IterationDocument:
        request = self.prepare_get_document(eval_id, dataset_id, iteration_id, document_id)
        response = await self._client._prepared_request(request)
        return IterationDocument.model_validate(response)

    async def list_documents(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        limit: int = 1000,
        offset: int = 0,
    ) -> List[IterationDocument]:
        request = self.prepare_list_documents(eval_id, dataset_id, iteration_id, limit=limit, offset=offset)
        response = await self._client._prepared_request(request)
        if isinstance(response, list):
            return [IterationDocument.model_validate(item) for item in response]
        return [IterationDocument.model_validate(item) for item in response.get("data", [])]

    async def update_document(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        document_id: str,
        prediction_data: Any | None = None,
        extraction_id: str | None = None,
    ) -> IterationDocument:
        request = self.prepare_update_document(eval_id, dataset_id, iteration_id, document_id, prediction_data=prediction_data, extraction_id=extraction_id)
        response = await self._client._prepared_request(request)
        return IterationDocument.model_validate(response)

    async def delete_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> dict:
        request = self.prepare_delete_document(eval_id, dataset_id, iteration_id, document_id)
        return await self._client._prepared_request(request)

    async def get_metrics(
        self,
        eval_id: str,
        dataset_id: str,
        iteration_id: str,
        force_refresh: bool = False,
    ) -> OptimizedIterationMetrics:
        request = self.prepare_get_metrics(eval_id, dataset_id, iteration_id, force_refresh=force_refresh)
        response = await self._client._prepared_request(request)
        return OptimizedIterationMetrics.model_validate(response)

    async def process_document(self, eval_id: str, dataset_id: str, iteration_id: str, document_id: str) -> RetabParsedChatCompletion:
        request = self.prepare_process_document(eval_id, dataset_id, iteration_id, document_id)
        response = await self._client._prepared_request(request)
        return RetabParsedChatCompletion.model_validate(response)



from .client import AsyncRetab, Retab, SignatureVerificationError
from . import utils
from . import types
from .exceptions import (
    APIConnectionError,
    APIError,
    APITimeoutError,
    AuthenticationError,
    InternalServerError,
    NotFoundError,
    PermissionDeniedError,
    RateLimitError,
    RetabError,
    ValidationError,
)
from .types.documents.extract import ExtractResponse, ExtractionResult, RetabParsedChatCompletion
from .types.documents.parse import ParseResponse
from .types.documents.classify import ClassifyResponse
from .types.documents.split import SplitResponse
from .types.documents.edit import EditResponse
from .types.extractions import Extraction, SourcesResponse
from .types.mime import MIMEData
from .types.pagination import PaginatedList

__all__ = [
    "Retab",
    "AsyncRetab",
    "SignatureVerificationError",
    "utils",
    "types",
    # Exceptions
    "RetabError",
    "APIError",
    "AuthenticationError",
    "PermissionDeniedError",
    "NotFoundError",
    "ValidationError",
    "RateLimitError",
    "InternalServerError",
    "APIConnectionError",
    "APITimeoutError",
    # Response types
    "Extraction",
    "ExtractResponse",
    "ExtractionResult",
    "RetabParsedChatCompletion",
    "ParseResponse",
    "ClassifyResponse",
    "SplitResponse",
    "EditResponse",
    "SourcesResponse",
    # Core types
    "MIMEData",
    "PaginatedList",
]



import datetime
from typing import Any, Optional

import nanoid  # type: ignore
from pydantic import AliasChoices, BaseModel, ConfigDict, Field, computed_field, field_validator

from ..documents.split import Subdocument
from ..inference_settings import InferenceSettings
from ..mime import BaseMIMEData, MIMEData
from ..projects.predictions import PredictionData
from ..projects.model import default_inference_settings


class SplitDraftConfig(BaseModel):
    model_config = ConfigDict(extra="ignore", populate_by_name=True)

    inference_settings: InferenceSettings = Field(default=default_inference_settings)
    split_config: list[Subdocument] = Field(
        default_factory=list,
        validation_alias=AliasChoices("split_config", "subdocuments"),
        serialization_alias="split_config",
    )
    json_schema: dict[str, Any] = Field(default_factory=dict)
    subdocuments: list[Subdocument] = Field(default_factory=list)


class SplitPublishedConfig(SplitDraftConfig):
    origin: str = Field(default="manual")


class SplitProject(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str = Field(default_factory=lambda: "project_" + nanoid.generate())
    name: str = Field(default="")
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    published_config: SplitPublishedConfig = Field(default_factory=SplitPublishedConfig)
    draft_config: SplitDraftConfig = Field(default_factory=SplitDraftConfig)
    is_published: bool = False


class CreateSplitProjectRequest(BaseModel):
    model_config = ConfigDict(extra="ignore", populate_by_name=True)

    name: str
    split_config: list[Subdocument] = Field(
        default_factory=list,
        validation_alias=AliasChoices("split_config", "subdocuments"),
        serialization_alias="split_config",
    )


class PatchSplitProjectRequest(BaseModel):
    model_config = ConfigDict(extra="ignore")

    name: Optional[str] = Field(default=None)
    published_config: Optional[SplitPublishedConfig] = Field(default=None)
    draft_config: Optional[SplitDraftConfig] = Field(default=None)
    is_published: Optional[bool] = Field(default=None)


class SplitDataset(BaseModel):
    model_config = ConfigDict(extra="ignore", populate_by_name=True)

    id: str
    name: str = Field(default="", description="The name of the dataset")
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    base_split_config: list[Subdocument] = Field(
        default_factory=list,
        validation_alias=AliasChoices("base_split_config", "base_json_schema", "subdocuments", "base_subdocuments"),
        serialization_alias="base_split_config",
    )
    base_json_schema: dict[str, Any] = Field(default_factory=dict)
    base_subdocuments: list[Subdocument] = Field(default_factory=list)
    base_inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)
    project_id: str


class CreateSplitDatasetRequest(BaseModel):
    model_config = ConfigDict(extra="ignore", populate_by_name=True)

    name: str
    base_split_config: list[Subdocument] = Field(
        default_factory=list,
        validation_alias=AliasChoices("base_split_config", "base_json_schema", "subdocuments", "base_subdocuments"),
        serialization_alias="base_split_config",
    )
    base_inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)


class PatchSplitDatasetRequest(BaseModel):
    model_config = ConfigDict(extra="ignore")

    name: Optional[str] = Field(default=None, description="The name of the dataset")


class SplitDatasetDocument(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    project_id: str
    dataset_id: str
    mime_data: BaseMIMEData = Field(description="The mime data of the document")
    prediction_data: PredictionData = Field(default_factory=PredictionData)
    extraction_id: str | None = Field(default=None)
    split_id: str | None = Field(default=None)
    validation_flags: dict[str, Any] = Field(default_factory=dict)


class SplitBuilderDocument(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    project_id: str
    mime_data: BaseMIMEData = Field(description="The mime data of the document")
    prediction_data: PredictionData = Field(default_factory=PredictionData)
    split_id: str | None = Field(default=None)
    extraction_id: str | None = Field(default=None)


class PatchSplitDatasetDocumentRequest(BaseModel):
    model_config = ConfigDict(extra="ignore")

    validation_flags: Optional[dict[str, Any]] = Field(default=None)
    prediction_data: Optional[PredictionData] = Field(default=None)
    extraction_id: Optional[str] = Field(default=None)


class SplitConfigOverrides(BaseModel):
    model_config = ConfigDict(extra="ignore", populate_by_name=True)

    descriptions_override: Optional[dict[str, str]] = Field(
        default=None,
        validation_alias=AliasChoices("descriptions_override", "descriptionsOverride"),
        serialization_alias="descriptions_override",
    )
    partition_keys_override: Optional[dict[str, str | None]] = Field(
        default=None,
        validation_alias=AliasChoices("partition_keys_override", "partitionKeysOverride"),
        serialization_alias="partition_keys_override",
    )


class SplitDraftIteration(BaseModel):
    model_config = ConfigDict(extra="ignore")

    split_config_overrides: SplitConfigOverrides = Field(
        default_factory=SplitConfigOverrides,
        validation_alias=AliasChoices("split_config_overrides", "config_overrides", "schema_overrides"),
        serialization_alias="split_config_overrides",
    )
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)


class SplitIteration(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)
    split_config_overrides: SplitConfigOverrides = Field(
        default_factory=SplitConfigOverrides,
        validation_alias=AliasChoices("split_config_overrides", "config_overrides", "schema_overrides"),
        serialization_alias="split_config_overrides",
    )
    parent_id: Optional[str] = Field(default=None)
    project_id: str
    dataset_id: str
    draft: SplitDraftIteration = Field(default_factory=SplitDraftIteration)
    status: str = Field(default="draft")
    finalized_at: datetime.datetime | None = Field(default=None)
    last_finalize_error: str | None = Field(default=None)

    @field_validator("status", mode="before")
    @classmethod
    def _normalize_status(cls, value: str | None) -> str:
        if value == "completed":
            return "finalized"
        return value or "draft"


class CreateSplitIterationRequest(BaseModel):
    model_config = ConfigDict(extra="ignore")

    inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)
    split_config_overrides: SplitConfigOverrides = Field(
        default_factory=SplitConfigOverrides,
        validation_alias=AliasChoices("split_config_overrides", "config_overrides", "schema_overrides"),
        serialization_alias="split_config_overrides",
    )
    project_id: str
    dataset_id: str
    parent_id: Optional[str] = Field(default=None)


class PatchSplitIterationRequest(BaseModel):
    model_config = ConfigDict(extra="ignore")

    inference_settings: Optional[InferenceSettings] = Field(default=None)
    split_config_overrides: Optional[SplitConfigOverrides] = Field(
        default=None,
        validation_alias=AliasChoices("split_config_overrides", "config_overrides", "schema_overrides"),
        serialization_alias="split_config_overrides",
    )
    draft: Optional[SplitDraftIteration] = Field(default=None)


class SplitIterationDocument(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    project_id: str
    iteration_id: str
    dataset_id: str
    dataset_document_id: str
    mime_data: BaseMIMEData = Field(description="The mime data of the document")
    prediction_data: PredictionData = Field(default_factory=PredictionData)
    extraction_id: str | None = Field(default=None)
    split_id: str | None = Field(default=None)



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict, Field
from pydantic import field_validator

from ..inference_settings import InferenceSettings
from ..mime import BaseMIMEData
from .predictions import PredictionData


class SchemaOverrides(BaseModel):
    model_config = ConfigDict(extra="ignore")

    descriptionsOverride: Optional[dict[str, str]] = Field(default=None, description="Maps field path to description override")
    reasoningPromptsOverride: Optional[dict[str, str]] = Field(default=None, description="Maps field path to X-ReasoningPrompt override")


class DraftIteration(BaseModel):
    model_config = ConfigDict(extra="ignore")

    schema_overrides: SchemaOverrides = Field(default_factory=SchemaOverrides)
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)


class Iteration(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    inference_settings: InferenceSettings = Field(default_factory=InferenceSettings)
    schema_overrides: SchemaOverrides = Field(default_factory=SchemaOverrides)
    parent_id: Optional[str] = Field(default=None, description="The ID of the parent iteration")
    project_id: str
    dataset_id: str
    draft: DraftIteration = Field(default_factory=DraftIteration, description="The drafted changes of the iteration")
    status: str = Field(default="draft", description="Iteration status: draft, finalizing, or finalized")

    @field_validator("status", mode="before")
    @classmethod
    def _normalize_status(cls, value: str | None) -> str:
        if value == "completed":
            return "finalized"
        return value or "draft"


class IterationDocument(BaseModel):
    model_config = ConfigDict(extra="ignore")

    id: str
    updated_at: datetime.datetime = Field(default_factory=lambda: datetime.datetime.now(tz=datetime.timezone.utc))
    project_id: str
    iteration_id: str
    dataset_id: str
    dataset_document_id: str
    mime_data: BaseMIMEData = Field(description="The mime data of the document")
    prediction_data: PredictionData = Field(default_factory=PredictionData, description="The prediction data of the document")
    extraction_id: str | None = Field(default=None, description="The extraction id of the document")




from typing import Any, Awaitable, Dict, Literal, TypeVar, get_args

import nanoid  # type: ignore
import pytest

from retab import AsyncRetab, Retab
from retab.types.projects.datasets import Dataset
from retab.types.projects.iterations import Iteration
from retab.types.inference_settings import InferenceSettings

T = TypeVar("T")

TEST_MODEL = "retab-micro"


async def await_or_return(obj: T | Awaitable[T]) -> T:
    if isinstance(obj, Awaitable):
        return await obj
    else:
        return obj


ClientType = Literal["sync", "async"]


@pytest.mark.asyncio
@pytest.mark.parametrize("client_type", get_args(ClientType))
async def test_dataset_crud(
    sync_client: Retab,
    async_client: AsyncRetab,
    client_type: ClientType,
    booking_confirmation_json_schema: Dict[str, Any],
) -> None:
    """Test basic CRUD operations for datasets."""
    client = sync_client if client_type == "sync" else async_client
    project_name = f"test_dataset_crud_{nanoid.generate()}"

    # Create a project first
    project = await await_or_return(
        client.evals.extract.create(name=project_name, json_schema=booking_confirmation_json_schema)
    )
    eval_id = project.id
    published_project = await await_or_return(client.evals.extract.publish(eval_id))
    assert published_project.id == eval_id

    try:
        # CREATE dataset
        dataset_name = f"dataset_{nanoid.generate()}"
        dataset = await await_or_return(
            client.evals.extract.datasets.create(
                eval_id=eval_id,
                name=dataset_name,
                base_json_schema=booking_confirmation_json_schema,
            )
        )
        assert isinstance(dataset, Dataset)
        assert dataset.name == dataset_name
        assert dataset.project_id == eval_id
        dataset_id = dataset.id

        try:
            # GET dataset
            retrieved = await await_or_return(
                client.evals.extract.datasets.get(eval_id=eval_id, dataset_id=dataset_id)
            )
            assert retrieved.id == dataset_id
            assert retrieved.name == dataset_name

            # LIST datasets
            datasets = await await_or_return(
                client.evals.extract.datasets.list(eval_id=eval_id)
            )
            assert any(d.id == dataset_id for d in datasets)

            # UPDATE dataset
            new_name = f"updated_{nanoid.generate()}"
            updated = await await_or_return(
                client.evals.extract.datasets.update(
                    eval_id=eval_id, dataset_id=dataset_id, name=new_name
                )
            )
            assert updated.name == new_name

        finally:
            # DELETE dataset
            try:
                await await_or_return(
                    client.evals.extract.datasets.delete(eval_id=eval_id, dataset_id=dataset_id)
                )
            except Exception:
                pass
    finally:
        # Clean up project
        try:
            await await_or_return(client.evals.extract.delete(eval_id))
        except Exception:
            pass


@pytest.mark.asyncio
@pytest.mark.parametrize("client_type", get_args(ClientType))
async def test_iteration_draft_workflow(
    sync_client: Retab,
    async_client: AsyncRetab,
    client_type: ClientType,
    booking_confirmation_json_schema: Dict[str, Any],
) -> None:
    """Test the iteration draft workflow: create → update_draft → get_schema → finalize."""
    client = sync_client if client_type == "sync" else async_client
    project_name = f"test_iteration_draft_{nanoid.generate()}"

    # Create project
    project = await await_or_return(
        client.evals.extract.create(name=project_name, json_schema=booking_confirmation_json_schema)
    )
    eval_id = project.id
    published_project = await await_or_return(client.evals.extract.publish(eval_id))
    assert published_project.id == eval_id

    try:
        # Create dataset
        dataset = await await_or_return(
            client.evals.extract.datasets.create(
                eval_id=eval_id,
                name=f"dataset_{nanoid.generate()}",
                base_json_schema=booking_confirmation_json_schema,
            )
        )
        dataset_id = dataset.id

        try:
            # CREATE iteration (starts as draft)
            iteration = await await_or_return(
                client.evals.extract.datasets.iterations.create(
                    eval_id=eval_id,
                    dataset_id=dataset_id,
                )
            )
            assert isinstance(iteration, Iteration)
            assert iteration.status == "draft"
            iteration_id = iteration.id

            try:
                # GET iteration
                retrieved = await await_or_return(
                    client.evals.extract.datasets.iterations.get(
                        eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id
                    )
                )
                assert retrieved.id == iteration_id
                assert retrieved.status == "draft"

                # LIST iterations
                iterations = await await_or_return(
                    client.evals.extract.datasets.iterations.list(
                        eval_id=eval_id, dataset_id=dataset_id
                    )
                )
                assert any(i.id == iteration_id for i in iterations)

                # UPDATE DRAFT with new inference settings
                new_settings = InferenceSettings(model="retab-small", n_consensus=1)
                updated = await await_or_return(
                    client.evals.extract.datasets.iterations.update_draft(
                        eval_id=eval_id,
                        dataset_id=dataset_id,
                        iteration_id=iteration_id,
                        inference_settings=new_settings,
                    )
                )
                assert updated.draft.inference_settings.model == "retab-small"

                # GET SCHEMA (without draft)
                schema_response = await await_or_return(
                    client.evals.extract.datasets.iterations.get_schema(
                        eval_id=eval_id,
                        dataset_id=dataset_id,
                        iteration_id=iteration_id,
                    )
                )
                assert "json_schema" in schema_response

                # GET SCHEMA (with draft applied)
                draft_schema_response = await await_or_return(
                    client.evals.extract.datasets.iterations.get_schema(
                        eval_id=eval_id,
                        dataset_id=dataset_id,
                        iteration_id=iteration_id,
                        use_draft=True,
                    )
                )
                assert "json_schema" in draft_schema_response

                # FINALIZE iteration (promotes draft → main, status → completed)
                finalized = await await_or_return(
                    client.evals.extract.datasets.iterations.finalize(
                        eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id
                    )
                )
                assert finalized.status in {"finalizing", "finalized"}
                assert finalized.inference_settings.model == "retab-small"

            finally:
                # DELETE iteration
                try:
                    await await_or_return(
                        client.evals.extract.datasets.iterations.delete(
                            eval_id=eval_id, dataset_id=dataset_id, iteration_id=iteration_id
                        )
                    )
                except Exception:
                    pass
        finally:
            try:
                await await_or_return(
                    client.evals.extract.datasets.delete(eval_id=eval_id, dataset_id=dataset_id)
                )
            except Exception:
                pass
    finally:
        try:
            await await_or_return(client.evals.extract.delete(eval_id))
        except Exception:
            pass



from pydantic import BaseModel, Field

class RetabUsage(BaseModel):
    """Usage information for document processing."""
    credits: float = Field(..., description="Credits consumed for processing")



from typing import Any, Optional

from pydantic import BaseModel

from ...types.projects import CreateProjectRequest, PatchProjectRequest, Project
from ...types.standards import DeleteResponse, PreparedRequest, UNSET, _Unset


class EvalCRUDMixin:
    BASE: str
    PROJECT_MODEL: type[BaseModel] = Project
    CREATE_MODEL: type[BaseModel] = CreateProjectRequest
    PATCH_MODEL: type[BaseModel] = PatchProjectRequest

    def prepare_create(
        self,
        name: str,
        json_schema: dict[str, Any],
        **extra_body: Any,
    ) -> PreparedRequest:
        eval_dict: dict[str, Any] = {
            "name": name,
            "json_schema": json_schema,
        }
        if extra_body:
            eval_dict.update(extra_body)

        eval_data = self.CREATE_MODEL(**eval_dict)
        return PreparedRequest(method="POST", url=self.BASE, data=eval_data.model_dump(exclude_unset=True, mode="json"))

    def prepare_get(self, eval_id: str) -> PreparedRequest:
        return PreparedRequest(method="GET", url=f"{self.BASE}/{eval_id}")

    def prepare_update(
        self,
        eval_id: str,
        name: str | _Unset = UNSET,
        json_schema: dict[str, Any] | _Unset = UNSET,
        **extra_body: Any,
    ) -> PreparedRequest:
        update_dict: dict[str, Any] = {}
        if name is not UNSET:
            update_dict["name"] = name
        if json_schema is not UNSET:
            update_dict["json_schema"] = json_schema
        if extra_body:
            update_dict.update(extra_body)

        data = self.PATCH_MODEL(**update_dict).model_dump(exclude_unset=True, mode="json")
        return PreparedRequest(method="PATCH", url=f"{self.BASE}/{eval_id}", data=data)

    def prepare_list(self, **extra_params: Any) -> PreparedRequest:
        params = extra_params or None
        return PreparedRequest(method="GET", url=self.BASE, params=params)

    def prepare_delete(self, eval_id: str) -> PreparedRequest:
        return PreparedRequest(method="DELETE", url=f"{self.BASE}/{eval_id}")

    def prepare_publish(self, eval_id: str, origin: Optional[str] = None) -> PreparedRequest:
        params = {"origin": origin} if origin else None
        return PreparedRequest(method="POST", url=f"{self.BASE}/{eval_id}/publish", params=params)


class SyncEvalCRUDMixin(EvalCRUDMixin):
    def create(
        self,
        name: str,
        json_schema: dict[str, Any],
        **extra_body: Any,
    ) -> Project:
        request = self.prepare_create(name, json_schema, **extra_body)
        response = self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    def get(self, eval_id: str) -> Project:
        request = self.prepare_get(eval_id)
        response = self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    def list(self, **extra_params: Any) -> list[Project]:
        request = self.prepare_list(**extra_params)
        response = self._client._prepared_request(request)
        return [self.PROJECT_MODEL.model_validate(item) for item in response.get("data", [])]

    def update(
        self,
        eval_id: str,
        name: str | _Unset = UNSET,
        json_schema: dict[str, Any] | _Unset = UNSET,
        **extra_body: Any,
    ) -> Project:
        request = self.prepare_update(eval_id, name=name, json_schema=json_schema, **extra_body)
        response = self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    def delete(self, eval_id: str) -> DeleteResponse:
        request = self.prepare_delete(eval_id)
        return self._client._prepared_request(request)

    def publish(self, eval_id: str, origin: Optional[str] = None) -> Project:
        request = self.prepare_publish(eval_id, origin=origin)
        response = self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)


class AsyncEvalCRUDMixin(EvalCRUDMixin):
    async def create(self, name: str, json_schema: dict[str, Any], **extra_body: Any) -> Project:
        request = self.prepare_create(name, json_schema, **extra_body)
        response = await self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    async def get(self, eval_id: str) -> Project:
        request = self.prepare_get(eval_id)
        response = await self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    async def list(self, **extra_params: Any) -> list[Project]:
        request = self.prepare_list(**extra_params)
        response = await self._client._prepared_request(request)
        return [self.PROJECT_MODEL.model_validate(item) for item in response.get("data", [])]

    async def update(
        self,
        eval_id: str,
        name: str | _Unset = UNSET,
        json_schema: dict[str, Any] | _Unset = UNSET,
        **extra_body: Any,
    ) -> Project:
        request = self.prepare_update(eval_id, name=name, json_schema=json_schema, **extra_body)
        response = await self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)

    async def delete(self, eval_id: str) -> DeleteResponse:
        request = self.prepare_delete(eval_id)
        return await self._client._prepared_request(request)

    async def publish(self, eval_id: str, origin: Optional[str] = None) -> Project:
        request = self.prepare_publish(eval_id, origin=origin)
        response = await self._client._prepared_request(request)
        return self.PROJECT_MODEL.model_validate(response)



import warnings
from io import IOBase
from pathlib import Path
from typing import Any, Dict, Optional

import PIL.Image
from pydantic import HttpUrl
from ...._resource import AsyncAPIResource, SyncAPIResource
from ....utils.mime import MIMEData
from ....utils.stream_context_managers import as_async_context_manager, as_context_manager
from ....types.documents.extract import RetabParsedChatCompletion, RetabParsedChatCompletionChunk
from ....types.standards import PreparedRequest
from .._base import AsyncEvalCRUDMixin, SyncEvalCRUDMixin
from .._multipart import build_process_request
from .datasets import Datasets, AsyncDatasets
from .templates import AsyncTemplates, Templates

BASE = "/evals/extract"


class ExtractMixin:
    BASE = BASE

    def prepare_process(
        self,
        eval_id: str,
        iteration_id: str | None = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> PreparedRequest:
        if "documents" in extra_form:
            raise TypeError("client.evals.extract.process(...) accepts only 'document'.")
        return build_process_request(
            base=self.BASE,
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )

    def prepare_extract(
        self,
        project_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
        ) -> PreparedRequest:
        return self.prepare_process(
            eval_id=project_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )

    def prepare_process_stream(
        self,
        eval_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> PreparedRequest:
        if "documents" in extra_form:
            raise TypeError("client.evals.extract.process_stream(...) accepts only 'document'.")
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        return request.model_copy(update={"url": f"{request.url}/stream"})


class Extract(SyncAPIResource, SyncEvalCRUDMixin, ExtractMixin):
    """Extract evals API."""

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.datasets = Datasets(client=client)
        self.templates = Templates(client=client)

    def process(
        self,
        eval_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> RetabParsedChatCompletion:
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        response = self._client._prepared_request(request)
        return RetabParsedChatCompletion.model_validate(response)

    def extract(
        self,
        project_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
        ) -> RetabParsedChatCompletion:
        warnings.warn(
            "client.evals.extract.extract(...) is deprecated; use client.evals.extract.process(...) instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.process(
            eval_id=project_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )

    @as_context_manager
    def process_stream(
        self,
        eval_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ):
        request = self.prepare_process_stream(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        for chunk_json in self._client._prepared_request_stream(request):
            if not chunk_json:
                continue
            yield RetabParsedChatCompletionChunk.model_validate(chunk_json)


class AsyncExtract(AsyncAPIResource, AsyncEvalCRUDMixin, ExtractMixin):
    """Async extract evals API."""

    def __init__(self, client: Any, **kwargs: Any) -> None:
        super().__init__(client=client, **kwargs)
        self.datasets = AsyncDatasets(client=client)
        self.templates = AsyncTemplates(client=client)

    async def process(
        self,
        eval_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ) -> RetabParsedChatCompletion:
        request = self.prepare_process(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        response = await self._client._prepared_request(request)
        return RetabParsedChatCompletion.model_validate(response)

    async def extract(
        self,
        project_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
        ) -> RetabParsedChatCompletion:
        warnings.warn(
            "client.evals.extract.extract(...) is deprecated; use client.evals.extract.process(...) instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return await self.process(
            eval_id=project_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )

    @as_async_context_manager
    async def process_stream(
        self,
        eval_id: str,
        iteration_id: Optional[str] = None,
        document: Path | str | bytes | IOBase | MIMEData | PIL.Image.Image | HttpUrl | None = None,
        model: str | None = None,
        image_resolution_dpi: int | None = None,
        n_consensus: int | None = None,
        metadata: Dict[str, str] | None = None,
        extraction_id: str | None = None,
        **extra_form: Any,
    ):
        request = self.prepare_process_stream(
            eval_id=eval_id,
            iteration_id=iteration_id,
            document=document,
            model=model,
            image_resolution_dpi=image_resolution_dpi,
            n_consensus=n_consensus,
            metadata=metadata,
            extraction_id=extraction_id,
            **extra_form,
        )
        async for chunk_json in self._client._prepared_request_stream(request):
            if not chunk_json:
                continue
            yield RetabParsedChatCompletionChunk.model_validate(chunk_json)


ExtractProjects = Extract
AsyncExtractProjects = AsyncExtract
